///////////////////
// Event Handler //
///////////////////

#include "events.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

const char COMMENT = '#';

BOOL SkipSpace (const char **ptr)
{
    while (1)
        switch (**ptr) {
            case ' ':
            case '\t':
            case '\n':
                (*ptr) ++;
                break;
            case NULL:
                return FALSE;
            default:
                return TRUE;
        }
}

BOOL CopyField (const char **ptr, char *buf)
{
    if (!SkipSpace (ptr)) {
        *buf = NULL;
        return FALSE;
    }

    while (1)
        switch (**ptr) {
            case ' ':
            case '\t':
            case '\n':
                (*ptr) ++;
            case NULL:
                *buf ++ = NULL;
                return TRUE;
            default:
                *buf ++ = *(*ptr)++;
                break;
        }
}

BOOL CopyRest (const char **ptr, char *buf)
{
    if (!SkipSpace (ptr)) {
        *buf = NULL;
        return FALSE;
    }

    while (1)
        switch (**ptr) {
            case '\n':
                (*ptr) ++;
            case NULL:
                *buf ++ = NULL;
                return TRUE;
            default:
                *buf ++ = *(*ptr)++;
                break;
        }
}

Events::Events ()
{
    events = new Event [MAXEVENTS];
    entries = 0;
}

Events::~Events ()
{
    delete events;
}

int Events::Load (const char *commands)
{
    FILE *file;
    Event event;
    char line [MAXLINE];
    char field [MAXLINE];
    char *ptr;

    Remove ();

    file = fopen (commands, "r");
    if (!file) return -1;

    while (fgets (line, sizeof (line), file)) {
        ptr = line;
        SkipSpace (&ptr);

        if (*ptr && (*ptr != COMMENT)) {

            CopyField (&ptr, field);
            event.minute = (strcmp (field,"*"))? atoi (field) : -1;
            CopyField (&ptr, field);
            event.hour = (strcmp (field,"*"))? atoi (field) : -1;
            CopyField (&ptr, field);
            event.day = (strcmp (field,"*"))? atoi (field) : -1;

            CopyField (&ptr, field);
            event.month = atoi (field);
            if (event.month == 0) {
                if (!strncmpi (field, "Jan", 3)) event.month = 1; else
                if (!strncmpi (field, "Feb", 3)) event.month = 2; else
                if (!strncmpi (field, "Mar", 3)) event.month = 3; else
                if (!strncmpi (field, "Apr", 3)) event.month = 4; else
                if (!strncmpi (field, "May", 3)) event.month = 5; else
                if (!strncmpi (field, "Jun", 3)) event.month = 6; else
                if (!strncmpi (field, "Jul", 3)) event.month = 7; else
                if (!strncmpi (field, "Aug", 3)) event.month = 8; else
                if (!strncmpi (field, "Sep", 3)) event.month = 9; else
                if (!strncmpi (field, "Oct", 3)) event.month = 10; else
                if (!strncmpi (field, "Nov", 3)) event.month = 11; else
                if (!strncmpi (field, "Dec", 3)) event.month = 12; else
                event.month = -1;
            }

            CopyField (&ptr, field);
            event.weekday = atoi (field);
            if (event.weekday == 0) {
                if (!strncmpi (field, "Sun", 3)) event.weekday = 0; else
                if (!strncmpi (field, "Mon", 3)) event.weekday = 1; else
                if (!strncmpi (field, "Tue", 3)) event.weekday = 2; else
                if (!strncmpi (field, "Wed", 3)) event.weekday = 3; else
                if (!strncmpi (field, "Thu", 3)) event.weekday = 4; else
                if (!strncmpi (field, "Fri", 3)) event.weekday = 5; else
                if (!strncmpi (field, "Sat", 3)) event.weekday = 6; else
                event.weekday = -1;
            } else
                event.weekday --;

            CopyRest (&ptr, event.exec);

            memset (&event.lastrun, 0, sizeof (event.lastrun));
            events [entries ++] = event;
        }
    }

    fclose (file);
    return 0;
}

void Events::Remove ()
{
    entries = 0;
}

int Events::Check (SYSTEMTIME& time)
{
    epp_Process process;
    Event event;
    int index;
    int run;

    run = 0;

    for (index = 0; index < entries; index ++) {
        event = events [index];

        // set wildcard fields to current
        if (event.minute == -1) event.minute = time.wMinute;
        if (event.hour == -1) event.hour = time.wHour;
        if (event.month == -1) event.month = time.wMonth;
        if ((event.day == -1) && (event.weekday == -1)) {
            event.day = time.wDay;
            event.weekday = time.wDayOfWeek;
        }

        // if current time and date
        if ((event.minute == time.wMinute) &&
           (event.hour == time.wHour) &&
           (event.month == time.wMonth)  &&
           ((event.day == time.wDay) ||
           (event.weekday == time.wDayOfWeek))) {

            process.Create (event.exec);
            process.Close ();

            events [index].lastrun = time;
            run ++;
        }
   }

    return run;
}